"""
======
rororo
======

Collection of utilities, helpers, and principles for building Python backend
applications. Supports `aiohttp.web <http://aiohttp.readthedocs.org/>`_,
`Flask <http://flask.pocoo.org/>`_, and your web-framework.

"""

__author__ = 'Igor Davydenko'
__license__ = 'BSD'
__version__ = '1.0.0a1'
